/* ignuit - Educational software for the GNOME, following the Leitner
 * flash-card system.
 *
 * Copyright (C) 2008 Timothy Musson
 *
 * <trmusson@ihug.co.nz>
 * http://homepages.ihug.co.nz/~trmusson/programs.html#ignuit
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <config.h>
#include <gnome.h>
#include <glade/glade.h>

#include "main.h"
#include "file.h"
#include "app-window.h"
#include "dialog-editor.h"
#include "dialog-find.h"


#define HISTORY_LENGTH   10


typedef struct {

    Ignuit              *ig;

    GladeXML            *gui;
    GtkWidget           *window;

    GtkWidget           *combo_entry;
    GSList              *face_group;

} DialogData;


static DialogData *dialog = NULL;


enum {
    RADIO_FRONTS = 0,
    RADIO_BACKS,
    RADIO_BOTH,
    N_RADIO_BUTTONS
};


static gint
get_radio_selection (DialogData *data)
{
    GSList *cur;
    gint i = N_RADIO_BUTTONS;

    for (cur = data->face_group; cur != NULL; cur = cur->next) {

        i--;

        if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(cur->data))) {
            return i;
        }
    }
    g_assert_not_reached ();
}


static void
cb_find (GtkWidget *widget, DialogData *data)
{
    GtkComboBox *combo;
    gboolean fronts, backs, hit;
    gchar *text;
    GList *cur, *cards;
    Category *cat;
    Card *c;

    combo = GTK_COMBO_BOX(data->combo_entry);

    text = gtk_combo_box_get_active_text (combo);
    if (text == NULL) {
        return;
    }

    dialog_editor_check_changed ();

    if (text[0] != '\0') {

        /* Add this query to search history. */

        gtk_combo_box_prepend_text (combo, text);
        gtk_combo_box_remove_text (combo, HISTORY_LENGTH);

        data->ig->recent_search_terms = g_list_prepend
            (data->ig->recent_search_terms, text);

        if (g_list_length (data->ig->recent_search_terms) > HISTORY_LENGTH) {

            GList *item;

            item = g_list_last (data->ig->recent_search_terms);
            data->ig->recent_search_terms = g_list_remove_link
                (data->ig->recent_search_terms, item);

            g_free (item->data);
        }
    }

    fronts = TRUE;
    backs = TRUE;

    switch (get_radio_selection (data)) {
    case RADIO_FRONTS:
        backs = FALSE;
        break;
    case RADIO_BACKS:
        fronts = FALSE;
        break;
    case RADIO_BOTH:
        break;
    }

    /* Clear any previous search results. */
    file_clear_search (data->ig->file, TRUE);

    for (cur = file_get_cards (data->ig->file); cur != NULL; cur = cur->next) {

        hit = FALSE;

        c = CARD(cur);

        cat = card_get_category (c);
        g_assert (cat != NULL);

        if (fronts) {
            if (text[0] == '\0') {
                hit = card_front_is_blank (c);
            }
            else {
                hit = (strstr (card_get_front (c), text) != NULL);
            }
        }

        if (backs) {
            if (text[0] == '\0') {
                hit |= card_back_is_blank (c);
            }
            else {
                hit |= (strstr (card_get_back (c), text) != NULL);
            }
        }

        if (hit) {
            file_add_search_card (data->ig->file, c);
        }

    }

    /* If text wasn't added to ig->recent_search_terms, free it now. */
    if (text[0] == '\0') {
        g_free (text);
    }

    cat = file_get_search (data->ig->file);
    cards = category_get_cards (cat);

    app_window_refresh_card_pane (data->ig, cards);
    app_window_select_category (data->ig, NULL);
    app_window_select_card (data->ig, cards);

    file_set_current_category (data->ig->file, cat);
    file_set_current_item (data->ig->file, cards);


    /* Update main window controls and appbar. */

    gtk_widget_set_sensitive (data->ig->m_remove_category, FALSE);
    gtk_widget_set_sensitive (data->ig->b_remove_category, FALSE);
    gtk_widget_set_sensitive (data->ig->m_add_card, FALSE);
    gtk_widget_set_sensitive (data->ig->b_add_card, FALSE);
    gtk_widget_set_sensitive (data->ig->m_switch_sides, cards != NULL);

    app_window_update_appbar (data->ig);


    dialog_editor_tweak ();
}


static void
cb_destroy (GtkWidget *widget, DialogData *data)
{
    g_object_unref (G_OBJECT(data->gui));
    g_free (data);

    dialog = NULL;
}


static void
cb_close (GtkWidget *widget, DialogData *data)
{
    gtk_widget_destroy (data->window);
}


void
dialog_find (Ignuit *ig)
{
    GtkWidget *w, *b_close, *b_find, *hbox, *label, *entry;
    gchar *gladefile;
    GList *cur;


    if (dialog != NULL) {
        gtk_window_present (GTK_WINDOW(dialog->window));
        return;
    }

    gladefile = gnome_program_locate_file (ig->program,
        GNOME_FILE_DOMAIN_APP_DATADIR, F_GLADE_FIND, TRUE, NULL);

    if (gladefile == NULL) {
        g_warning ("Can't find file: %s\n", F_GLADE_FIND);
        return;
    }

    dialog = g_new0 (DialogData, 1);
    dialog->gui = glade_xml_new (gladefile, NULL, NULL);
    g_free (gladefile);

    dialog->ig = ig;

    dialog->window = glade_xml_get_widget (dialog->gui, "dialog");

    label = glade_xml_get_widget (dialog->gui, "label_search");
    dialog->combo_entry = glade_xml_get_widget (dialog->gui, "combo_entry");

    b_close = glade_xml_get_widget (dialog->gui, "b_close");
    b_find = glade_xml_get_widget (dialog->gui, "b_find");

    hbox = glade_xml_get_widget (dialog->gui, "hbox1");
    dialog->combo_entry = gtk_combo_box_entry_new_text ();
    gtk_box_pack_start (GTK_BOX(hbox), dialog->combo_entry, TRUE, TRUE, 0);
    gtk_widget_show (dialog->combo_entry);

    gtk_label_set_mnemonic_widget (GTK_LABEL(label), dialog->combo_entry);

    for (cur = ig->recent_search_terms; cur != NULL; cur = cur->next) {
        gtk_combo_box_append_text (GTK_COMBO_BOX(dialog->combo_entry),
            (gchar*)cur->data);
    }

    entry = gtk_bin_get_child (GTK_BIN(dialog->combo_entry));

    w = glade_xml_get_widget (dialog->gui, "r_fronts");
    dialog->face_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON(w));

    w = glade_xml_get_widget (dialog->gui, "r_both");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), TRUE);

    g_signal_connect (G_OBJECT(dialog->window), "destroy",
        G_CALLBACK(cb_destroy), dialog);

    g_signal_connect (G_OBJECT(entry), "activate",
        G_CALLBACK(cb_find), dialog);
    g_signal_connect (G_OBJECT(b_close), "clicked",
        G_CALLBACK(cb_close), dialog);
    g_signal_connect (G_OBJECT(b_find), "clicked",
        G_CALLBACK(cb_find), dialog);

    gtk_window_set_transient_for (GTK_WINDOW(dialog->window),
        GTK_WINDOW(ig->app));

    gtk_widget_show (dialog->window);
}

