/* ignuit - Educational software for the GNOME, following the Leitner
 * flash-card system.
 *
 * Copyright (C) 2008 Timothy Musson
 *
 * <trmusson@ihug.co.nz>
 * http://homepages.ihug.co.nz/~trmusson/programs.html#ignuit
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <config.h>
#include <gnome.h>
#include <gconf/gconf-client.h>

#include "main.h"
#include "prefs.h"


struct _Prefs {

    GConfClient *gconf_client;

    gint        app_width;
    gint        app_height;
    gint        quiz_width;
    gint        quiz_height;
    gint        category_pane_width;
    gint        category_column_title_width;
    gint        card_column_front_width;
    gboolean    sticky_flips;
    gchar       *font_small;
    gchar       *font_large;
    gchar       *workdir;
    GdkColor    *color[N_CARD_COLORS];
    gint        schedule[N_GROUPS];

};


#ifndef gdk_color_to_string

/* gdk_color_to_string exists in gdk >= 2.12 */

#define gdk_color_to_string(c)  color_to_string(c)

static gchar*
color_to_string (GdkColor* color)
{
    return g_strdup_printf ("#%04X%04X%04X",
        color->red, color->green, color->blue);
}
#endif


void
prefs_csvstr_to_schedules (Prefs *p, const gchar *csvstr)
{
    gchar **v;
    gint i;


    /* Set schedules from a string of comma separated values. */

    v = g_strsplit (csvstr, ",", N_GROUPS);
    for (i = 0; i < N_GROUPS; i++) {
        if (v[i] == NULL) {
            break;
        }
        prefs_set_schedule (p, i, atoi (v[i]));
    }
    g_strfreev (v);
}


static gchar*
prefs_schedules_to_csvstr (Prefs *p)
{
    gchar *s, *v[N_GROUPS + 1];
    gint i;


    /* Return a newly allocated string of comma separated schedules. */

    for (i = 0; i < N_GROUPS; i++) {
        v[i] = g_strdup_printf ("%d", p->schedule[i]);
    }
    v[N_GROUPS] = NULL;

    s = g_strjoinv (",", v);

    for (i = 0; i < N_GROUPS; i++) {
        g_free (v[i]);
    }

    return s;
}


void
prefs_csvstr_to_colors (Prefs *p, const gchar *csvstr)
{
    gchar **v;
    gint i;


    /* Set card colours from a string of comma separated values. */

    v = g_strsplit (csvstr, ",", N_CARD_COLORS);
    for (i = 0; i < N_CARD_COLORS; i++) {
        if (v[i] == NULL) {
            break;
        }
        prefs_set_color (p, i, v[i]);
    }
    g_strfreev (v);
}


static gchar*
prefs_colors_to_csvstr (Prefs *p)
{
    gchar *s, *v[N_CARD_COLORS + 1];
    gint i;


    /* Return a newly allocated string of comma separated colour values. */

    for (i = 0; i < N_CARD_COLORS; i++) {
        g_assert (p->color[i] != NULL);
        v[i] = gdk_color_to_string (p->color[i]);
    }
    v[N_CARD_COLORS] = NULL;

    s = g_strjoinv (",", v);

    for (i = 0; i < N_CARD_COLORS; i++) {
        g_free (v[i]);
    }

    return s;
}


void
prefs_set_sticky_flips (Prefs *p, gboolean sticky)
{
    if (sticky != p->sticky_flips) {
        p->sticky_flips = sticky;
        gconf_client_set_bool (p->gconf_client,
            PREF_STICKY_FLIPS, sticky, NULL);
    }
}


gboolean
prefs_get_sticky_flips (Prefs *p)
{
    return p->sticky_flips;
}


void
prefs_set_category_pane_width (Prefs *p, gint width)
{
    if (width <= 0) { width = DEFAULT_CATEGORY_PANE_WIDTH; }

    if (width != p->category_pane_width) {
        p->category_pane_width = width;
        gconf_client_set_int (p->gconf_client,
            PREF_CATEGORY_PANE_WIDTH, width, NULL);
    }
}


gint
prefs_get_category_pane_width (Prefs *p)
{
    return p->category_pane_width;
}


void
prefs_set_color_gdk (Prefs *p, Color which, GdkColor *color)
{
    gboolean changed = FALSE;

    if (p->color[which]) {
        changed = !gdk_color_equal (color, p->color[which]);
        gdk_color_free (p->color[which]);
    }

    p->color[which] = gdk_color_copy (color);

    if (changed) {

        gchar *spec;

        spec = prefs_colors_to_csvstr (p);
        gconf_client_set_string (p->gconf_client, PREF_CARD_COLORS, spec, NULL);
        g_free (spec);
    }
}


void
prefs_set_color (Prefs *p, Color which, const gchar *spec)
{
    GdkColor color;

    if (gdk_color_parse (spec, &color)) {
        prefs_set_color_gdk (p, which, &color);
    }
}


GdkColor*
prefs_get_color (Prefs *p, Color which)
{
    return p->color[which];
}


Group
prefs_get_schedule (Prefs *p, Group g)
{
    return p->schedule[g];
}


void
prefs_set_schedule (Prefs *p, Group g, gint days)
{
    if (days != p->schedule[g]) {

        gchar *s;

        p->schedule[g] = days;

        s = prefs_schedules_to_csvstr (p);
        gconf_client_set_string (p->gconf_client, PREF_SCHEDULES, s, NULL);
        g_free (s);
    }
}


void
prefs_set_app_size (Prefs *p, gint width, gint height)
{
    if (width < 1) { width = DEFAULT_APP_WIDTH; }
    if (height < 1) { height = DEFAULT_APP_HEIGHT; }

    if (width != p->app_width) {
        p->app_width = width;
        gconf_client_set_int (p->gconf_client,
            PREF_APP_WIDTH, width, NULL);
    }
    if (height != p->app_height) {
        p->app_height = height;
        gconf_client_set_int (p->gconf_client,
            PREF_APP_HEIGHT, height, NULL);
    }
}


gint
prefs_get_app_width (Prefs *p)
{
    return p->app_width;
}


gint
prefs_get_app_height (Prefs *p)
{
    return p->app_height;
}


void
prefs_set_quiz_size (Prefs *p, gint width, gint height)
{
    if (width < 1) { width = DEFAULT_QUIZ_WIDTH; }
    if (height < 1) { height = DEFAULT_QUIZ_HEIGHT; }

    if (width != p->quiz_width) {
        p->quiz_width = width;
        gconf_client_set_int (p->gconf_client,
            PREF_QUIZ_WIDTH, width, NULL);
    }
    if (height != p->quiz_height) {
        p->quiz_height = height;
        gconf_client_set_int (p->gconf_client,
            PREF_QUIZ_HEIGHT, height, NULL);
    }
}


gint
prefs_get_quiz_width (Prefs *p)
{
    return p->quiz_width;
}


gint
prefs_get_quiz_height (Prefs *p)
{
    return p->quiz_height;
}


void
prefs_set_category_column_title_width (Prefs *p, gint width)
{
    if (width <= 0) { width = DEFAULT_CATEGORY_COLUMN_TITLE_WIDTH; }

    if (width != p->category_column_title_width) {
        p->category_column_title_width = width;
        gconf_client_set_int (p->gconf_client,
            PREF_CATEGORY_COLUMN_TITLE_WIDTH, width, NULL);
    }
}


gint
prefs_get_category_column_title_width (Prefs *p)
{
    return p->category_column_title_width;
}


void
prefs_set_card_column_front_width (Prefs *p, gint width)
{
    if (width <= 0) { width = DEFAULT_CARD_COLUMN_FRONT_WIDTH; }

    if (width != p->card_column_front_width) {
        p->card_column_front_width = width;
        gconf_client_set_int (p->gconf_client,
            PREF_CARD_COLUMN_FRONT_WIDTH, width, NULL);
    }
}


gint
prefs_get_card_column_front_width (Prefs *p)
{
    return p->card_column_front_width;
}


void
prefs_set_font_small (Prefs *p, const gchar *fontname)
{
    if (p->font_small == NULL ||
        (g_ascii_strcasecmp (p->font_small, fontname) != 0)) {

            g_free (p->font_small);
            p->font_small = g_strdup (fontname);

            gconf_client_set_string (p->gconf_client,
                PREF_FONT_SMALL, fontname, NULL);
    }
}


const gchar*
prefs_get_font_small (Prefs *p)
{
    return p->font_small;
}


void
prefs_set_font_large (Prefs *p, const gchar *fontname)
{
    if (p->font_large == NULL ||
        (g_ascii_strcasecmp (p->font_large, fontname) != 0)) {

            g_free (p->font_large);
            p->font_large = g_strdup (fontname);

            gconf_client_set_string (p->gconf_client,
                PREF_FONT_LARGE, fontname, NULL);
    }
}


const gchar*
prefs_get_font_large (Prefs *p)
{
    return p->font_large;
}


void
prefs_set_workdir (Prefs *p, const gchar *dirname)
{
    if (p->workdir == NULL || (strcmp (p->workdir, dirname) != 0)) {

        g_free (p->workdir);
        p->workdir = g_strdup (dirname);

        gconf_client_set_string (p->gconf_client, PREF_WORKDIR, dirname, NULL);
    }
}


const gchar*
prefs_get_workdir (Prefs *p)
{
    return p->workdir;
}


Prefs*
prefs_load (void)
{
    gint w, h;
    gboolean b;
    gchar *s;
    Prefs *p;

    p = g_new0(Prefs, 1);

    p->gconf_client = gconf_client_get_default ();

    w = gconf_client_get_int (p->gconf_client, PREF_APP_WIDTH, NULL);
    h = gconf_client_get_int (p->gconf_client, PREF_APP_HEIGHT, NULL);
    prefs_set_app_size (p, w, h);

    w = gconf_client_get_int (p->gconf_client, PREF_CATEGORY_PANE_WIDTH, NULL);
    prefs_set_category_pane_width (p, w);

    w = gconf_client_get_int (p->gconf_client, PREF_CATEGORY_COLUMN_TITLE_WIDTH, NULL);
    prefs_set_category_column_title_width (p, w);

    w = gconf_client_get_int (p->gconf_client, PREF_CARD_COLUMN_FRONT_WIDTH, NULL);
    prefs_set_card_column_front_width (p, w);

    b = gconf_client_get_bool (p->gconf_client, PREF_STICKY_FLIPS, NULL);
    prefs_set_sticky_flips (p, b);

    w = gconf_client_get_int (p->gconf_client, PREF_QUIZ_WIDTH, NULL);
    h = gconf_client_get_int (p->gconf_client, PREF_QUIZ_HEIGHT, NULL);
    prefs_set_quiz_size (p, w, h);

    s = gconf_client_get_string (p->gconf_client, PREF_FONT_SMALL, NULL);
    prefs_set_font_small (p, s ? s : DEFAULT_FONT_SMALL);
    g_free (s);

    s = gconf_client_get_string (p->gconf_client, PREF_FONT_LARGE, NULL);
    prefs_set_font_large (p, s ? s : DEFAULT_FONT_LARGE);
    g_free (s);

    s = gconf_client_get_string (p->gconf_client, PREF_CARD_COLORS, NULL);
    prefs_csvstr_to_colors (p, s ? s : DEFAULT_CARD_COLORS);
    g_free (s);

    s = gconf_client_get_string (p->gconf_client, PREF_SCHEDULES, NULL);
    prefs_csvstr_to_schedules (p, s ? s : DEFAULT_SCHEDULES);
    g_free (s);

    if ((s = gconf_client_get_string (p->gconf_client, PREF_WORKDIR, NULL))) {
        prefs_set_workdir (p, s);
    }

    return p;
}


void
prefs_free (Prefs *p)
{
    gint i;

    g_free (p->font_small);
    g_free (p->font_large);
    g_free (p->workdir);

    for (i = 0; i < N_CARD_COLORS; i++) {
        if (p->color[i]) {
            gdk_color_free (p->color[i]);
        }
    }

    g_free (p);
}

